% Compare active basis model with SVM per category:
%
% 1. compare the feature weights before/after svm tuning
% 2. compare the binary classification accuracy

load('Split/categoryInfo.mat','categories');
load('Split/Natural/split.mat','negTestImages');
nC = length(categories);
overwrite = true;

for iRun = 11
    % sketch template
	symbolicTemplates = cell(1,nC);
	
	for iCat = 1:nC
		% load sketch template
		load(sprintf('Result/%s/ActiveBasisLocalNormSVM/model_run%d.mat', categories{iCat}, iRun), 'S2Template');
		%sym = displayGaborTemplate(size(sym), Mx(1:nSketchToDisplay),...
		%	My(1:nSketchToDisplay), Mi(1:nSketchToDisplay), Mm(1:nSketchToDisplay) );
		sym = S2Template.sym;
		nsk = sum( S2Template.selectedGain > 0.2 );
		
		% load texture template
		load(sprintf('Result/%s/ActiveBasisLocalNormSVM/model_run%d.mat', categories{iCat}, iRun), 'S2TemplateTexture'...
			);
		ntex = sum( S2TemplateTexture.selectedGain > 0.2 );
		
		% load svm model
		
		svmModelFile = sprintf( 'Result/%s/ActiveBasisLocalNormSVM/svmmodel_run%d.txt', categories{iCat}, iRun );
		fid = fopen(svmModelFile, 'r');
		
		% read line 10: number of SV
		for line = 1:9
			tline = fgetl(fid);
		end
		nSV = fscanf(fid, '%d', 1) - 1;
		disp(sprintf('nSV=%d',nSV));
		tline = fgetl(fid);
		tline = fgetl(fid);
		
		D = nsk + ntex; % the dimension of feature vector
		disp(sprintf('feature length = %d',D));
		
		lambdas = zeros(1,D);
		
		% read line 12~end: SV's
		for jSV = 1:nSV
			% read alpha * y
			alpha = fscanf( fid, '%f', 1 );
			vec = zeros(1,D);
			for d = 1:D
				feaID = fscanf( fid, '%d', 1 );
				colon = fscanf( fid, '%c', 1 );
				feaVal = fscanf( fid, '%f', 1 );
				vec(d) = feaVal;
			end
			lambdas = lambdas + vec * alpha;
			tline = fgetl(fid);
		end
		
		%
		% visualize the sketch template
		%
		
		% before svm
		sym = displayGaborTemplate(size(sym), S2Template.selectedRow(1:nsk)+ 1 + floor(size(sym,1)/2),...
			S2Template.selectedCol(1:nsk)+ 1 + floor(size(sym,1)/2), S2Template.selectedOri(1:nsk), S2Template.selectedLambda(1:nsk) );
		
		imfile = sprintf( 'Result/%s/ActiveBasisLocalNormSVM/sketch_template_before_svm_run%d.png', categories{iCat}, iRun );
		imwrite( sym, imfile );
		
		% after svm
		sym = displayGaborTemplate(size(sym), S2Template.selectedRow(1:nsk) + 1 + floor(size(sym,1)/2),...
			S2Template.selectedCol(1:nsk) + 1 + floor(size(sym,1)/2), S2Template.selectedOri(1:nsk), max(0,lambdas(1:nsk)) );
			
		imfile = sprintf( 'Result/%s/ActiveBasisLocalNormSVM/sketch_template_after_svm_run%d.png', categories{iCat}, iRun );
		imwrite( sym, imfile );	
	end
	
	%
	% compare binary classification accuracy
	%
	load(sprintf('%s/svmresult_run%d.mat','Result/summary/ActiveBasisLocalNormSVM', iRun), 'svm_prediction');
	load(sprintf('%s/eval_result_run%d.mat','Result/summary/ActiveBasisLocalNormSVM', iRun), 'tssMat');
	numImage = 0;
    for iCat = 1:nC
        load(sprintf('Split/%s/split%d.mat',categories{iCat},iRun),'posTestImages');
        pos{iCat} = numImage + (1 : length(posTestImages));
        numImage = numImage + length(posTestImages);
    end
	% 
	for iCat = 1:nC
		mode = 1;
		tssPos = {svm_prediction(pos{iCat},iCat)};
		tssNeg = {svm_prediction(setdiff(1:numImage,pos{iCat}),iCat)};
		[ap testAUC tp fp rec prec EER] = evaluate(tssPos,tssNeg,mode);
		svm_perf{iCat} = struct( 'ap', ap, 'textAUC', testAUC, 'tp', tp, 'fp', fp, 'rec', rec, 'prec', prec, 'EER', EER );
		tssPos = {tssMat(pos{iCat},iCat)};
		tssNeg = {tssMat(setdiff(1:numImage,pos{iCat}),iCat)};
		[ap testAUC tp fp rec prec EER] = evaluate(tssPos,tssNeg,mode);
		activebasis_perf{iCat} = struct( 'ap', ap, 'textAUC', testAUC, 'tp', tp, 'fp', fp, 'rec', rec, 'prec', prec, 'EER', EER );
	end
	
	filename = sprintf( '%s/compare_result_run%d.mat','Result/summary/ActiveBasisLocalNormSVM', iRun );
	save(filename, 'svm_perf', 'activebasis_perf');
end
