clear
%% Deal with background (negative) images

% load negative images
negImageNames = dir('Image/Natural/*.jpg');

% split the data into half  (training set and testing set)
nTrain = floor(length(negImageNames)/2); nTest = length(negImageNames) - nTrain;
perm = randperm(length(negImageNames));
negTrainImages = cell(1,nTrain);
for i = 1:nTrain
    negTrainImages{i} = negImageNames(perm(i)).name;
end
negTestImages = cell(1,nTest);
for i = 1:nTest
    negTestImages{i} = negImageNames(perm(i+nTrain)).name;
end
if ~exist(sprintf('Split/%s','Natural'),'dir')
    mkdir( sprintf('Split/%s', 'Natural' ) );
end
save(sprintf('Split/%s/split.mat','Natural'),...
        'negTrainImages','negTestImages');


%% Deal with positive images

dirnames = dir('Image/');
categories = cell(length(dirnames),1);
for i = 1:length(categories)
    categories{i} = dirnames(i).name;
end
categories = setdiff(categories,{'Natural','.','..'}); % remove the category of natural images 

% number of categories
nCat = length(categories);

nPosTrainArray = [5 10 15 20 30 50];
iRunStart = [1 6 11 16 21 26];

%
% For each category (excluding the background category), randomly select
%   some examples.
%

% run 1~5: #trainpos = 5
% run 6~10: #trainpos = 10
% ...

% collect all positive image paths
for iCat = 1:nCat
    disp(categories{iCat});
    % load positive images
    imgNames = dir(sprintf('Image/%s/*.jpg',categories{iCat}));
    if isempty(imgNames)
        imgNames = dir(sprintf('Image/%s/*.png',categories{iCat}));
    end
    if isempty(imgNames)
        imgNames = dir(sprintf('Image/%s/*.bmp',categories{iCat}));
    end

    posImageNames = cell(length(imgNames),1);
    for i = 1:length(imgNames)
        posImageNames{i} = sprintf('%s',imgNames(i).name);
    end

    for iTrainSize = 1:length(iRunStart)
        nPosTrain = nPosTrainArray(iTrainSize);
        if nPosTrain >= length(posImageNames)
            disp('nPosTrain > length(posImageNames)');
            continue;
        end
        for iRun = iRunStart(iTrainSize)-1 + (1:5)
            % randomly assign nPosTrain positive images to training
            perm = randperm(length(posImageNames));
            posTrainImages = cell(1,nPosTrain);
            for i = 1:nPosTrain
                posTrainImages{i} = posImageNames{perm(i)};
            end
            % assign the rest positive images to testing
            posTestImages = cell(1,length(posImageNames)-nPosTrain);
            for i = 1:length(posTestImages)
                posTestImages{i} = posImageNames{perm(i+nPosTrain)};
            end
            if ~exist(sprintf('Split/%s',categories{iCat}),'dir')
                mkdir( sprintf('Split/%s',categories{iCat}) );
            end
            save(sprintf('Split/%s/split%d.mat',categories{iCat},iRun),...
                'posTrainImages','posTestImages');
            %disp(sprintf('saved: Split/%s/split%d.mat',categories{iCat},iRun));
        end
    end
end

save('Split/categoryInfo.mat','categories','nCat');

